@@ -17,3 +17,10 @@ def test_escape_special_characters_in_key():
     expected = '"foo\\nbar" = "baz"\n'
     assert expected == dumps(d)
     assert loads(dumps(d))["foo\nbar"] == "baz"
+
+
+def test_write_inline_table_in_nested_arrays():
+    d = {"foo": [[{"a": 1}]]}
+    expected = "foo = [[{a = 1}]]\n"
+    assert expected == dumps(d)
+    assert loads(dumps(d))["foo"] == [[{"a": 1}]]
